from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction

from codelist.models import KonovaCode
from codelist.settings import CODELIST_LAW_ID, CODELIST_PROCESS_TYPE_ID, CODELIST_HANDLER_ID, \
    CODELIST_CONSERVATION_OFFICE_ID, CODELIST_REGISTRATION_OFFICE_ID
from compensation.models import Payment
from intervention.models import Intervention, InterventionDocument, Legal, Handler, Responsibility
from konova.management.commands.kspMigrater.base_migrater import BaseMigrater


class InterventionMigrater(BaseMigrater):

    def _migrate_intervention_responsibility(self, intervention, eiv):
        intervention.responsible = intervention.responsible or Responsibility.objects.create()
        intervention.responsible.handler = intervention.responsible.handler or Handler.objects.create()
        eiv_reg_off = eiv[7]
        eiv_cons_off = eiv[9]
        eiv_handler_type = eiv[11]
        eiv_handler_detail = eiv[12]

        if eiv_reg_off is not None and eiv_reg_off != 0:
            try:
                reg_office_code = KonovaCode.objects.get(
                    atom_id=eiv_reg_off,
                    is_leaf=True,
                    code_lists__in=[CODELIST_REGISTRATION_OFFICE_ID],
                )
                intervention.responsible.registration_office = reg_office_code
            except ObjectDoesNotExist:
                intervention.comment = f"{intervention.comment or ''}\nNicht migrierbare Zulassungsbehörde: {eiv_reg_off}"
        intervention.responsible.registration_file_number = eiv[8]

        if eiv_cons_off is not None and eiv_cons_off != 0:
            try:
                cons_office_code = KonovaCode.objects.get(
                    atom_id=eiv_cons_off,
                    is_leaf=True,
                    code_lists__in=[CODELIST_CONSERVATION_OFFICE_ID],
                )
            except ObjectDoesNotExist:
                intervention.comment = f"{intervention.comment or ''}\nNicht migrierbare Eintragungsstelle: {eiv_cons_off}"
            intervention.responsible.conservation_office = cons_office_code
        intervention.responsible.conservation_file_number = eiv[10]

        if eiv_handler_type is not None and eiv_handler_type != 0:
            try:
                handler_type_code = KonovaCode.objects.get(
                    atom_id=eiv_handler_type,
                    is_leaf=True,
                    code_lists__in=[CODELIST_HANDLER_ID]
                )
                intervention.responsible.handler.type = handler_type_code
            except ObjectDoesNotExist:
                intervention.comment = f"{intervention.comment or ''}\nNicht migrierbarer Eingriffsverursacher_TYP: {eiv_handler_type}"
            intervention.responsible.handler.detail = eiv_handler_detail
            intervention.responsible.handler.save()

        intervention.responsible.save()
        return intervention

    def _migrate_intervention_legal(self, intervention, eiv):
        intervention.legal = intervention.legal or Legal.objects.create()
        eiv_process_type = eiv[5]
        eiv_law = eiv[6]
        eiv_date_registration = eiv[14]
        eiv_date_binding = eiv[15]

        if eiv_process_type is not None and eiv_process_type != 0:
            process_type_code = KonovaCode.objects.get(
                atom_id=eiv_process_type,
                is_leaf=True,
                code_lists__in=[CODELIST_PROCESS_TYPE_ID],
            )
            intervention.legal.process_type = process_type_code
        if eiv_law is not None and eiv_law != 0:
            law_code = KonovaCode.objects.get(
                atom_id=eiv_law,
                is_leaf=True,
                code_lists__in=[CODELIST_LAW_ID],
            )
            intervention.legal.laws.add(law_code)

        if eiv_date_registration is not None:
            intervention.legal.registration_date = eiv_date_registration
        if eiv_date_binding is not None:
            intervention.legal.binding_date = eiv_date_binding

        intervention.legal.save()
        return intervention

    def _migrate_intervention_payment(self, intervention, eiv):
        payment_date = eiv[16]
        payment_amount = eiv[17]

        if payment_amount is not None and payment_amount != 0:
            payment_exists = intervention.payments.filter(
                amount=payment_amount
            ).exists()
            if payment_exists:
                return intervention

            payment = Payment(
                amount=payment_amount,
            )
            if payment_date is None:
                payment.comment = "Datenmigration: Kein Zahlungsdatum hinterlegt! Schnellstmöglich nachtragen oder diesen Kommentar mit einer Begründung ersetzen, falls kein Datum existiert."
            else:
                payment.due_on = payment_date
            payment.save()
            intervention.payments.add(payment)
        return intervention

    def migrate(self):
        cursor = self.db_connection.cursor()
        cursor.execute(
            'select '
                'om."KENNUNG", '
                'linf."OBJBEZ", '
                'ST_AsEWKT(ST_Multi(ST_CollectionExtract(ST_MakeValid(ST_Transform(geomf.the_geom,4326)), 3))) as geomf, '
                'ST_AsEWKT(ST_Multi(ST_CollectionExtract(ST_MakeValid(ST_Transform(geoml.the_geom,4326)), 2))) as geoml, '
                'ST_AsEWKT(ST_Multi(ST_CollectionExtract(ST_MakeValid(ST_Transform(geomp.the_geom,4326)), 1))) as geomp, '
                'eiv.verfahrenstyp, '
                'vr.verfahrensrecht, '
                'adr.behoerde as zb, '
                'linf."AZ" as zb_az, '
                'adr.adr_pruef as ets, '
                'auf."AZ" as ets_az, '
                'adr.adressrolle as eingriffsverursacher_typ, '
                'adr."Bemerkung" as eingriffsverursacher_bemerkung, '
                'linf."Bemerkung" as eiv_comment, '
                'zt.erlass as Zulassungsdatum, '
                'zt.rechtskraft as Bestandskraftdatum, '
                'zt.baubeginn as ersatzzahlungstermin, '
                'eiv.ersatzzahlung '
            'from "OBJ_MASTER" om '
                'left join "LINFOS" linf on om."GISPADID"=linf."GISPADID" '
                'left join eiv on om."GISPADID"=eiv.gispadid '
                'left join verfahrensrecht vr on om."GISPADID"=vr.gispadid '
                'left join geometry_f geomf on om."GISPADID"=geomf.gispadid '
                'left join geometry_l geoml on om."GISPADID"=geoml.gispadid '
                'left join geometry_p geomp on om."GISPADID"=geomp.gispadid '
                'left join adressrolle adr on om."GISPADID"=adr."GISPADID" '
                'left join "Aufwertung" auf on om."GISPADID"=auf."GISPADID" '
                'left join zulassungstermin zt on eiv.zulassung=zt.id '
            'where '
                'om."OKL"=7730085 and '
                'om.archiv=false and '
                'om.nicht_vollstaendig=0;'
        )

        all_eivs = cursor.fetchall()
        len_all_eivs = len(all_eivs)
        num_processed = 0
        print(f"Migrate EIVs to interventions...")
        print(f"--Found {len_all_eivs} entries. Process now...")
        for eiv in all_eivs:
            if num_processed % 500 == 0:
                print(f"----{num_processed}/{len_all_eivs} processed")
            with transaction.atomic():
                eiv_comment = eiv[13] or ""
                intervention = Intervention.objects.get_or_create(
                    identifier=eiv[0]
                )[0]
                intervention.title = eiv[1]
                intervention.comment = eiv_comment

                intervention = self._migrate_geometry(intervention, eiv)
                intervention = self._migrate_intervention_legal(intervention, eiv)
                intervention = self._migrate_intervention_responsibility(intervention, eiv)
                intervention = self._migrate_intervention_payment(intervention, eiv)
                intervention = self._migrate_documents(intervention, InterventionDocument, eiv)

                intervention.save()
                num_processed += 1

        cursor.close()
